#!/usr/bin/python -d

import os
import sys

try:
    import rpm404
    rpm=rpm404
except:
    import rpm

class rpmHeader:
	def getHeader(self,rpm_file):
		fd = os.open(rpm_file, os.O_RDONLY)
		(header, isSource) = rpm.headerFromPackage(fd)
		os.close(fd)
		return (header,isSource)

	# Returns a list of entities this package obsoletes. Always returns a list.
	def obs_returnPkgdef(self,header):
		onames=self.returnObsname(header)
		if onames == None: return []

		onames = header[rpm.RPMTAG_OBSOLETENAME]
		overs = header[rpm.RPMTAG_OBSOLETEVERSION]
		oflags = header[rpm.RPMTAG_OBSOLETEFLAGS]

		if type(oflags) != type([]):
			oflags = [oflags]

		obslts=[]
		oindex=0

		print "Processing obsoletes list for %s-%s.%s.rpm"%(header[rpm.RPMTAG_NAME],header[rpm.RPMTAG_VERSION],header[rpm.RPMTAG_RELEASE])

		for oname in onames:
			obs = []
			obs.append (header[rpm.RPMTAG_NAME])
			obs.append (header[rpm.RPMTAG_VERSION])
			obs.append (header[rpm.RPMTAG_RELEASE])
			if header[rpm.RPMTAG_EPOCH]:
				obs.append ("%s" % header[rpm.RPMTAG_EPOCH])
			else:
				obs.append ("")
			obs.append (header[rpm.RPMTAG_ARCH])
      
			obs.append (oname)
			obs.append (overs[oindex])
			obs.append ("%s" % oflags[oindex])
      
			oindex = oindex + 1
			obslts.append (obs)

		if type(obslts) != type([]):
			return [obslts]
		else:
			return obslts

	def isObsempty(self,header):
		if self.returnObsname(header) != None:
			return 1
		return 0

	def returnObsname(self,header):
		return header[rpm.RPMTAG_OBSOLETENAME]

	def returnObsver(self,header):
		return header[rpm.RPMTAG_OBSOLETEVERSION]

	def returnObsflag(self,header):
		return header[rpm.RPMTAG_OBSOLETEFLAGS]

	def returnPkgdef(self,header):
		pkg=[]
		pkg.append (header[rpm.RPMTAG_NAME])
		pkg.append (header[rpm.RPMTAG_VERSION])
		pkg.append (header[rpm.RPMTAG_RELEASE])
		if header[rpm.RPMTAG_EPOCH]:
			pkg.append ("%s" % header[rpm.RPMTAG_EPOCH])
		else:
			pkg.append ("")
		pkg.append (header[rpm.RPMTAG_ARCH])
		return pkg

	def checkRpmMd5(self,header_file):
		# This code is copied from rpmUtils/up2date 2.8
		# RPM spews to stdout/stderr.  Redirect.
		saveStdout = os.dup(1)
		saveStderr = os.dup(2)
		redirStdout = os.open("/dev/null", os.O_WRONLY | os.O_APPEND)
		redirStderr = os.open("/dev/null", os.O_WRONLY | os.O_APPEND)
		os.dup2(redirStdout, 1)
		os.dup2(redirStderr, 2)
		# now do the rpm thing
		ret = rpm.checksig(header_file, rpm.CHECKSIG_MD5)
		# restore normal stdout and stderr
		os.dup2(saveStdout, 1)
		os.dup2(saveStderr, 2)
		# Clean up
		os.close(redirStdout)
		os.close(redirStderr)
		os.close(saveStdout)
		os.close(saveStderr)
		return ret

	def readHeaderBlob(blob):
		# This code is copied from rpmUtils/up2date 2.8
		# Read two unsigned int32
		i0, i1 = struct.unpack("!2I", blob[:8])
		if len(blob) != i0 * 16 + i1 + 8:
			# Corrupt header
			return None
		# if this header is corrupt, rpmlib exits and we stop ;-<
		try:
			hdr = rpm.headerLoad(blob)
		except:
			return None

		# Header successfully read
		return hdr

	def Exists(self,rpm_file):
		header_file=rpm_file[:-3]
		header_file=header_file+"hdr"
		return os.path.isfile(header_file)

	def Create(self,argv):
		bad_rpms = 0
		bad_headers = 0
		rpms_processed = 0
		headers_found = 0
		headers_generated = 0

		for rpm_file in argv:
			header_file=rpm_file[:-3]
			header_file=header_file+"hdr"
			print "Processing ", rpm_file
	
			if not self.checkRpmMd5(rpm_file):
				rpms_processed = rpms_processed + 1
				fd = os.open(rpm_file, 0)
				try:
					(hdr,isSource) = rpm.headerFromPackage(fd)
				except:
					os.close(fd)
					print "Error reading header"
				os.close(fd)
	
				if (not (os.access(header_file, os.R_OK))):
					print "Generating header ", header_file
					fd = os.open(header_file, os.O_WRONLY|os.O_CREAT, 0644)
	
					os.write(fd, hdr.unload())
					os.close(fd)
					headers_generated = headers_generated + 1
				else:
					fd = open(header_file, "r")
					# if this header is corrupt, rpmlib exits and we stop (most of the time) ;-<
					try:
						hdr = self.readHeaderBlob(fd.read())
						fd.close()
						headers_found = headers_found + 1
					except:
						fd.close()
						print "Bad header present :", header_file,", deleting file"
						bad_headers = bad_headers + 1
						os.unlink(header_file)
						print "Generating header ", header_file
						fd = os.open(header_file, os.O_WRONLY|os.O_CREAT, 0644)
	
						os.write(fd, hdr.unload())
						os.close(fd)
						headers_generated = headers_generated + 1
			else:
				print "Bad MD5 check on ", rpm_file
				bad_rpms = bad_rpms + 1
	
		print
		print "Processing complete"
		print
		print rpms_processed, "valid rpms were processed"
		print bad_rpms, "rpm files were found bad"
		print headers_found, "valid headers were processed"
		print headers_generated, "headers were generated"
		print bad_headers, "rpm headers were found bad"

# We want to do this in case someone calls this on its own
if __name__ == '__main__':
	if len(sys.argv) == 1:
		print "You have to specify RPM name(s) as argument"
		sys.exit(1)
	else:
		rpmHeader().Create(sys.argv[1:])
#		for rpm_file in sys.argv[1:]:
#			rpmHeader().Create(rpm_file)

