#!/usr/bin/python
#
# This is a part of the NRH-up2date package, licensed under the GPL.
#
# A helper script for nrh-repository-update. Reprocesses the package
# lists, and saves them in proper format. 
#
# Usage - for internal use of NRH-up2date
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version
# 2 of the License, or (at your option) any later version.

import sys, os, re, time 
import xmlrpclib
import getopt
try:
    import rpm404
    rpm=rpm404
except:
    import rpm
from nrh_header_lib import *

class process_pkglist:
	def run(self,localFilename,targetFilename):
		if not os.access(localFilename, os.R_OK):
		    print "Source file does not exist"
		    sys.exit(1)
		
		f = open(localFilename, "r")
		filecontents = f.read()
		
		try:
		    tmp_args, tmp_method = xmlrpclib.loads(filecontents)
		except:
		    # if there was an error decoding return as if we didnt find it
		    # the odd thing is, in testing, it's actually pretty hard to get
		    # a file to show as corrupt, I think perhaps the xmlrpclib parser
		    # is a bit too lenient...
		    print "Source file is corrupt"
		    sys.exit(1)
		
		f = open(targetFilename, "w")
		f.write (xmlrpclib.dumps((tmp_args[0],),methodresponse=1))
		f.close
		os.chmod (targetFilename,0644)

class generate_pkglist:
	def run(self,listname):
		dirList = os.listdir('./')
		dirList.sort()
		
		pkgs = []
		obs = []
		
		for file in dirList:
		    if re.search('\.rpm$', file):
			rpmhd=rpmHeader()
			(header,isSource)=rpmhd.getHeader(file)

			if not isSource:
				print "Processing", file
				pkg=rpmhd.returnPkgdef(header)
				pkg.append ("%s" % (os.stat(file)[6]))
				pkg.append(listname)
	
				pkgs.append (pkg)

				obslts=rpmhd.obs_returnPkgdef(header)

				for obsoletepackage in obslts:
					obs.append(obsoletepackage)

		currentTime = time.strftime('%Y%m%d%H%M%S',time.localtime(time.time()))
		
		f = open("%s.%s" % (listname,currentTime), "w")
		f.write (xmlrpclib.dumps((pkgs,),methodresponse=1))
		f.close
		
		f = open("%s-obsoletes.%s" % (listname,currentTime), "w")
		f.write (xmlrpclib.dumps((obs,),methodresponse=1))
		f.close

		return currentTime

if __name__ == '__main__':
	if len(sys.argv) < 2:
		print "You must pass in at least one argument"
		print "Example 1: nrh_pkglist_op -d [directory name] --generate redhat-linux-i386-7.3"
		print "Example 2: nrh_pkglist_op -d [directory name] --process redhat-linux-i386-7.3.[timestamp] --target listPackages/[timestamp]"
		print "If -d is omitted, this script assumes current directory"
		sys.exit(1)

	GENERATE=0
	PROCESS=0
	TARGET=0
	DIRECTORY="."
	channel=""
	optlist,arg=getopt.getopt(sys.argv[1:],'d:',["generate=","process=","target="])
	for i in optlist:
		if i[0] == "--generate":
			GENERATE=1
			channel=i[1]
		if i[0] == "--process":
			PROCESS=1
			localFilename=i[1]
		if i[0] == "--target":
			TARGET=1
			targetFilename=i[1]
		if i[0] == "-d": DIRECTORY=i[1]

	if (GENERATE and PROCESS) or (not GENERATE and not PROCESS):
		print "--generate and --process cannot be specified together or not specified at all"
		print "Exiting..."
		sys.exit(1)
	if (PROCESS and not TARGET) or (TARGET and not PROCESS):
		print "Both --process and --target argument must be given, if one is specified"
		print "Exiting..."
		sys.exit(1)
	if (GENERATE and channel==""):
		print "With --generate option, you must also specify channel name"
		print "Exiting..."
		sys.exit(1)

	try:
		os.chdir(DIRECTORY)
	except:
		print "Cannot chdir to %s\nExiting..."%DIRECTORY

	if GENERATE: generate_pkglist().run(channel)
	if PROCESS: process_pkglist().run(localFilename,targetFilename)
